package com.famoco.androidpcsclibdemo;

import android.content.Context;
import android.os.Bundle;
import androidx.appcompat.app.AppCompatActivity;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.famoco.androidpcsclib.ATR;
import com.famoco.androidpcsclib.Card;
import com.famoco.androidpcsclib.CardTerminal;
import com.famoco.androidpcsclib.CommandAPDU;
import com.famoco.androidpcsclib.PCSC;
import com.famoco.androidpcsclib.ResponseAPDU;
import com.famoco.androidpcsclib.Utils;

import java.util.ArrayList;
import java.util.List;

public class MainActivity extends AppCompatActivity implements PCSC.PCSCCallbackInterface {

    private static final String TAG = MainActivity.class.getSimpleName();
    private static final Boolean DEBUG = true;

    private Context mContext;

    // UI elements
    private Spinner mSpinnerReaders;
    private TextView mStatus;
    private TextView mAtr;
    private TextView mLogTerminal;
    private EditText mEditTextApdu;
    private Button mButtonSend;

    private PCSC mPCSC = null;
    private CardTerminal mSelectedTerminal;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        if (DEBUG) {
            Log.d(TAG, "onCreate");
        }
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        mContext = this;
        mSpinnerReaders = findViewById(R.id.spinnerReaders);
        mStatus = findViewById(R.id.status);
        mAtr = findViewById(R.id.atr);
        mEditTextApdu = findViewById(R.id.editTextApdu);
        mButtonSend = findViewById(R.id.buttonSend);
        mLogTerminal = findViewById(R.id.logTerminal);
        mLogTerminal.setMovementMethod(new ScrollingMovementMethod());

        mSpinnerReaders.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parentView, View selectedItemView, int position, long id) {
                if (DEBUG) {
                    Log.d(TAG, "onItemSelected: " + position);
                }
                mSelectedTerminal = mPCSC.getTerminals().get(position);
                onCardStatusChange(mSelectedTerminal, mSelectedTerminal.isCardPresent());
            }

            @Override
            public void onNothingSelected(AdapterView<?> parentView) {
                Log.i(TAG, "onNothingSelected");
            }

        });
    }

    @Override
    public void onResume() {
        super.onResume();
        if (DEBUG) {
            Log.d(TAG, "onResume");
        }

        mButtonSend.setEnabled(false);
        mAtr.setText(null);
        mStatus.setText(R.string.card_not_present);

        if (mPCSC == null) {
            Log.i(TAG, "Creating SC_PCSC ");
            mPCSC = new PCSC(this);
            mPCSC.enableCCIDLogs(true);
        }

        if (mPCSC.getTerminals().isEmpty()) {
            Log.i(TAG, "Registering devices ");
            mPCSC.init();
            if (mPCSC.getTerminals().isEmpty()) {
                Log.i(TAG, "No reader found");
                Toast.makeText(this, "No reader found", Toast.LENGTH_SHORT).show();
            } else {
                Log.i(TAG, "Reader found");
                // Update terminals list
                onTerminalListChange(mPCSC.getTerminals());
                // Force to update the selected terminal
                onCardStatusChange(mSelectedTerminal, mSelectedTerminal.isCardPresent());
            }
        }
    }

    @Override
    public void onPause() {
        if (DEBUG) {
            Log.d(TAG, "onPause");
        }
        // Release resources
        if (mPCSC != null) {
            mPCSC.close();
        }
        super.onPause();
    }

    @Override
    public void onTerminalListChange(List<CardTerminal> cardTerminals) {
        Log.i(TAG, "onTerminalListChange");
        final List<String> cardTerminalsNames = new ArrayList<>();
        for (CardTerminal cardTerminal:cardTerminals) {
            cardTerminalsNames.add(cardTerminal.getName());
        }
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                ArrayAdapter<String> adapter = new ArrayAdapter<>(mContext,
                        android.R.layout.simple_spinner_item, cardTerminalsNames);
                mSpinnerReaders.setAdapter(adapter);
                if (mSelectedTerminal == null && !cardTerminalsNames.isEmpty()) {
                    // Force to select the first terminal by default
                    mSelectedTerminal = mPCSC.getTerminals().get(0);
                }
                if (cardTerminalsNames.isEmpty()) {
                    mSelectedTerminal = null;
                    mStatus.setText(R.string.card_not_present);
                    mAtr.setText(null);
                    mButtonSend.setEnabled(false);
                }
            }
        });

    }

    @Override
    public void onCardStatusChange(final CardTerminal cardTerminal, final Boolean isCardPresent) {
        Log.i(TAG, "onCardStatusChange " + cardTerminal.getName() + " " + isCardPresent);
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                String currentCardTerminalName = (String)mSpinnerReaders.getSelectedItem();
                if (currentCardTerminalName.equals(cardTerminal.getName())) {
                    if (isCardPresent) {
                        mStatus.setText(R.string.card_present);
                        ATR atr = cardTerminal.getATR();
                        if (atr != null && atr.getBytes().length > 0) {
                            mAtr.setText(cardTerminal.getATR().toString());
                        } else {
                            mAtr.setText(null);
                        }
                        mButtonSend.setEnabled(true);
                    } else {
                        mStatus.setText(R.string.card_not_present);
                        mButtonSend.setEnabled(false);
                        mAtr.setText(null);
                    }
                }
            }
        });
    }

    /* UI events */
    public void onClearTerminal(View v) {
        Log.i(TAG, "onClearTerminal");
        mLogTerminal.setText("");
    }

    public void onSendApdu(final View v) {
        Log.i(TAG, "onSendApdu");
        final Card card = mSelectedTerminal.connect("*");
        final CommandAPDU commandAPDU;
        String hexCmd = mEditTextApdu.getText().toString();
        try {
            commandAPDU = new CommandAPDU(Utils.hexStringToByteArray(hexCmd.replaceAll("\\s+",
                    "")));
            mLogTerminal.append("> " + Utils.bytesToHex(commandAPDU.getBytes()) + "\n");
            new Thread(new Runnable() {
                public void run() {
                    final ResponseAPDU response = card.getBasicChannel().transmit(commandAPDU);
                    v.post(new Runnable() {
                        public void run() {
                            mLogTerminal.append("< " + Utils.bytesToHex(response.getBytes()) +
                                    "\n");
                            if (DEBUG) {
                                Log.d(TAG, "" + response.toString());
                            }
                        }
                    });
                }
            }).start();
        } catch (IllegalArgumentException | StringIndexOutOfBoundsException e) {
            Log.e(TAG, "Invalid APDU", e);
            Toast.makeText(this, "Invalid APDU", Toast.LENGTH_SHORT).show();
        }
    }
}
